@@ -239,16 +239,14 @@ impl Parser {
 
     pub fn parse_function(&mut self, id: SQLIdent) -> Result<ASTNode, ParserError> {
         self.expect_token(&Token::LParen)?;
-        if self.consume_token(&Token::RParen) {
-            Ok(ASTNode::SQLFunction {
-                id: id,
-                args: vec![],
-            })
+        let args = if self.consume_token(&Token::RParen) {
+            vec![]
         } else {
             let args = self.parse_expr_list()?;
             self.expect_token(&Token::RParen)?;
-            Ok(ASTNode::SQLFunction { id, args })
-        }
+            args
+        };
+        Ok(ASTNode::SQLFunction { id, args })
     }
 
     pub fn parse_case_expression(&mut self) -> Result<ASTNode, ParserError> {
@@ -328,7 +326,7 @@ impl Parser {
                         })
                     } else {
                         parser_err!(format!(
-                            "Expected IN or LIKE after NOT, found {:?}",
+                            "Expected BETWEEN, IN or LIKE after NOT, found {:?}",
                             self.peek_token()
                         ))
                     }
@@ -620,6 +618,8 @@ impl Parser {
         } else if self.parse_keyword("MATERIALIZED") || self.parse_keyword("VIEW") {
             self.prev_token();
             self.parse_create_view()
+        } else if self.parse_keyword("EXTERNAL") {
+            self.parse_create_external_table()
         } else {
             parser_err!(format!(
                 "Unexpected token after CREATE: {:?}",
@@ -628,6 +628,26 @@ impl Parser {
         }
     }
 
+    pub fn parse_create_external_table(&mut self) -> Result<SQLStatement, ParserError> {
+        self.expect_keyword("TABLE")?;
+        let table_name = self.parse_object_name()?;
+        let columns = self.parse_columns()?;
+        self.expect_keyword("STORED")?;
+        self.expect_keyword("AS")?;
+        let file_format = self.parse_identifier()?.parse::<FileFormat>()?;
+
+        self.expect_keyword("LOCATION")?;
+        let location = self.parse_literal_string()?;
+
+        Ok(SQLStatement::SQLCreateTable {
+            name: table_name,
+            columns,
+            external: true,
+            file_format: Some(file_format),
+            location: Some(location),
+        })
+    }
+
     pub fn parse_create_view(&mut self) -> Result<SQLStatement, ParserError> {
         let materialized = self.parse_keyword("MATERIALIZED");
         self.expect_keyword("VIEW")?;
@@ -650,62 +670,74 @@ impl Parser {
     pub fn parse_create_table(&mut self) -> Result<SQLStatement, ParserError> {
         let table_name = self.parse_object_name()?;
         // parse optional column list (schema)
-        let mut columns = vec![];
-        if self.consume_token(&Token::LParen) {
-            loop {
-                match self.next_token() {
-                    Some(Token::SQLWord(column_name)) => {
-                        let data_type = self.parse_data_type()?;
-                        let is_primary = self.parse_keywords(vec!["PRIMARY", "KEY"]);
-                        let is_unique = self.parse_keyword("UNIQUE");
-                        let default = if self.parse_keyword("DEFAULT") {
-                            let expr = self.parse_default_expr(0)?;
-                            Some(expr)
-                        } else {
-                            None
-                        };
-                        let allow_null = if self.parse_keywords(vec!["NOT", "NULL"]) {
-                            false
-                        } else if self.parse_keyword("NULL") {
-                            true
-                        } else {
-                            true
-                        };
-                        debug!("default: {:?}", default);
+        let columns = self.parse_columns()?;
 
-                        columns.push(SQLColumnDef {
-                            name: column_name.as_sql_ident(),
-                            data_type: data_type,
-                            allow_null,
-                            is_primary,
-                            is_unique,
-                            default,
-                        });
-                        match self.next_token() {
-                            Some(Token::Comma) => {}
-                            Some(Token::RParen) => {
-                                break;
-                            }
-                            other => {
-                                return parser_err!(format!(
-                                    "Expected ',' or ')' after column definition but found {:?}",
-                                    other
-                                ));
-                            }
-                        }
-                    }
-                    unexpected => {
-                        return parser_err!(format!("Expected column name, got {:?}", unexpected));
-                    }
-                }
-            }
-        }
         Ok(SQLStatement::SQLCreateTable {
             name: table_name,
             columns,
+            external: false,
+            file_format: None,
+            location: None,
         })
     }
 
+    fn parse_columns(&mut self) -> Result<Vec<SQLColumnDef>, ParserError> {
+        let mut columns = vec![];
+        if !self.consume_token(&Token::LParen) {
+            return Ok(columns);
+        }
+
+        loop {
+            match self.next_token() {
+                Some(Token::SQLWord(column_name)) => {
+                    let data_type = self.parse_data_type()?;
+                    let is_primary = self.parse_keywords(vec!["PRIMARY", "KEY"]);
+                    let is_unique = self.parse_keyword("UNIQUE");
+                    let default = if self.parse_keyword("DEFAULT") {
+                        let expr = self.parse_default_expr(0)?;
+                        Some(expr)
+                    } else {
+                        None
+                    };
+                    let allow_null = if self.parse_keywords(vec!["NOT", "NULL"]) {
+                        false
+                    } else if self.parse_keyword("NULL") {
+                        true
+                    } else {
+                        true
+                    };
+                    debug!("default: {:?}", default);
+
+                    columns.push(SQLColumnDef {
+                        name: column_name.as_sql_ident(),
+                        data_type,
+                        allow_null,
+                        is_primary,
+                        is_unique,
+                        default,
+                    });
+                    match self.next_token() {
+                        Some(Token::Comma) => {}
+                        Some(Token::RParen) => {
+                            break;
+                        }
+                        other => {
+                            return parser_err!(format!(
+                                "Expected ',' or ')' after column definition but found {:?}",
+                                other
+                            ));
+                        }
+                    }
+                }
+                unexpected => {
+                    return parser_err!(format!("Expected column name, got {:?}", unexpected));
+                }
+            }
+        }
+
+        Ok(columns)
+    }
+
     pub fn parse_table_key(&mut self, constraint_name: SQLIdent) -> Result<TableKey, ParserError> {
         let is_primary_key = self.parse_keywords(vec!["PRIMARY", "KEY"]);
         let is_unique_key = self.parse_keywords(vec!["UNIQUE", "KEY"]);
@@ -1320,6 +1352,7 @@ impl Parser {
     /// Parse a restricted `SELECT` statement (no CTEs / `UNION` / `ORDER BY`),
     /// assuming the initial `SELECT` was already consumed
     pub fn parse_select(&mut self) -> Result<SQLSelect, ParserError> {
+        let distinct = self.parse_keyword("DISTINCT");
         let projection = self.parse_select_list()?;
 
         let (relation, joins) = if self.parse_keyword("FROM") {
@@ -1349,6 +1382,7 @@ impl Parser {
         };
 
         Ok(SQLSelect {
+            distinct,
             projection,
             selection,
             relation,
